@@ -38,24 +38,19 @@
 #include "nm-utils/nm-shared-utils.h"
 
 
-#define INLINE_BLOB_CA                  "ca"
-#define INLINE_BLOB_CERT                "cert"
-#define INLINE_BLOB_KEY                 "key"
-#define INLINE_BLOB_PKCS12              "pkcs12"
-#define INLINE_BLOB_SECRET              "secret"
-#define INLINE_BLOB_TLS_AUTH            "tls-auth"
-#define INLINE_BLOB_TLS_CRYPT           "tls-crypt"
+#define INLINE_BLOB_CA                  NMV_OVPN_TAG_CA
+#define INLINE_BLOB_CERT                NMV_OVPN_TAG_CERT
+#define INLINE_BLOB_EXTRA_CERTS         NMV_OVPN_TAG_EXTRA_CERTS
+#define INLINE_BLOB_KEY                 NMV_OVPN_TAG_KEY
+#define INLINE_BLOB_PKCS12              NMV_OVPN_TAG_PKCS12
+#define INLINE_BLOB_SECRET              NMV_OVPN_TAG_SECRET
+#define INLINE_BLOB_TLS_AUTH            NMV_OVPN_TAG_TLS_AUTH
+#define INLINE_BLOB_TLS_CRYPT           NMV_OVPN_TAG_TLS_CRYPT
 
 const char *_nmovpn_test_temp_path = NULL;
 
 /*****************************************************************************/
 
-static const char *
-_arg_is_set (const char *value)
-{
-	return (value && value[0]) ? value : NULL;
-}
-
 static void
 _auto_free_gstring_p (GString **ptr)
 {
@@ -939,6 +934,29 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CRL_VERIFY)) {
+			const char *file;
+			gs_free char *file_free = NULL;
+
+			if (!args_params_check_nargs_minmax (params, 1, 2, &line_error))
+				goto handle_line_error;
+			if (!args_params_check_arg_nonempty (params, 1, NULL, &line_error))
+				goto handle_line_error;
+			if (params[2] && !nm_streq (params[2], "dir")) {
+				line_error = g_strdup_printf (_("unsupported crl-verify argument"));
+				goto handle_line_error;
+			}
+
+			file = params[1];
+			if (!g_path_is_absolute (file))
+				file = file_free = g_build_filename (default_path, file, NULL);
+
+			if (params[2])
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_DIR, file);
+			else
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_FILE, file);
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_NS_CERT_TYPE)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
@@ -959,6 +977,17 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0],
+		                  NMV_OVPN_TAG_CONNECT_TIMEOUT,
+		                  NMV_OVPN_TAG_SERVER_POLL_TIMEOUT)) {
+			if (!args_params_check_nargs_n (params, 1, &line_error))
+				goto handle_line_error;
+			if (!args_params_parse_int64 (params, 1, 0, G_MAXINT, &v_int64, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item_int64 (s_vpn, NM_OPENVPN_KEY_CONNECT_TIMEOUT, v_int64);
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_FRAGMENT)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
@@ -1177,13 +1206,14 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		}
 
 		if (NM_IN_STRSET (params[0],
-		                 NMV_OVPN_TAG_PKCS12,
-		                 NMV_OVPN_TAG_CA,
-		                 NMV_OVPN_TAG_CERT,
-		                 NMV_OVPN_TAG_KEY,
-		                 NMV_OVPN_TAG_SECRET,
-		                 NMV_OVPN_TAG_TLS_AUTH,
-		                 NMV_OVPN_TAG_TLS_CRYPT)) {
+		                  NMV_OVPN_TAG_CA,
+		                  NMV_OVPN_TAG_CERT,
+		                  NMV_OVPN_TAG_EXTRA_CERTS,
+		                  NMV_OVPN_TAG_KEY,
+		                  NMV_OVPN_TAG_PKCS12,
+		                  NMV_OVPN_TAG_SECRET,
+		                  NMV_OVPN_TAG_TLS_AUTH,
+		                  NMV_OVPN_TAG_TLS_CRYPT)) {
 			const char *file;
 			gs_free char *file_free = NULL;
 			gboolean can_have_direction;
@@ -1214,6 +1244,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_KEY, file);
 			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CA))
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CA, file);
+			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_EXTRA_CERTS))
+				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_EXTRA_CERTS, file);
 			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CERT))
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CERT, file);
 			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_KEY))
@@ -1430,6 +1462,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				key = NM_OPENVPN_KEY_CERT;
 			else if (nm_streq (token, INLINE_BLOB_KEY))
 				key = NM_OPENVPN_KEY_KEY;
+			else if (nm_streq (token, INLINE_BLOB_EXTRA_CERTS))
+				key = NM_OPENVPN_KEY_EXTRA_CERTS;
 			else if (nm_streq (token, INLINE_BLOB_PKCS12)) {
 				is_base64 = TRUE;
 				key = NULL;
@@ -1739,7 +1773,7 @@ args_write_line_setting_value_int (GString *f,
 	nm_assert (setting_key && setting_key[0]);
 
 	value = nm_setting_vpn_get_data_item (s_vpn, setting_key);
-	if (!_arg_is_set (value))
+	if (!nmovpn_arg_is_set (value))
 		return;
 
 	v = _nm_utils_ascii_str_to_int64 (value, 10, G_MININT64, G_MAXINT64, 0);
@@ -1757,7 +1791,7 @@ args_write_line_setting_value (GString *f,
 	const char *value;
 
 	value = nm_setting_vpn_get_data_item (s_vpn, setting_key);
-	if (_arg_is_set (value))
+	if (nmovpn_arg_is_set (value))
 		args_write_line (f, tag_key, value);
 }
 
@@ -1797,7 +1831,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		return NULL;
 	}
 
-	gateways = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE));
+	gateways = nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE));
 	if (!gateways) {
 		g_set_error_literal (error,
 		                     NMV_EDITOR_PLUGIN_ERROR,
@@ -1806,7 +1840,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		return NULL;
 	}
 
-	connection_type = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE));
+	connection_type = nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE));
 
 	f = g_string_sized_new (512);
 
@@ -1849,18 +1883,18 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		                                   NM_OPENVPN_CONTYPE_PASSWORD,
 		                                   NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
-			if (_arg_is_set (value))
+			if (nmovpn_arg_is_set (value))
 				cacert = nm_utils_str_utf8safe_unescape (value, &cacert_free);
 		}
 
 		if (NM_IN_STRSET (connection_type, NM_OPENVPN_CONTYPE_TLS,
 		                                   NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
-			if (_arg_is_set (value))
+			if (nmovpn_arg_is_set (value))
 				user_cert = nm_utils_str_utf8safe_unescape (value, &user_cert_free);
 
 			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-			if (_arg_is_set (value))
+			if (nmovpn_arg_is_set (value))
 				private_key = nm_utils_str_utf8safe_unescape (value, &private_key_free);
 		}
 
@@ -1884,13 +1918,13 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	if (NM_IN_STRSET (connection_type, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY);
-		if (_arg_is_set (value)) {
+		if (nmovpn_arg_is_set (value)) {
 			gs_free char *s_free = NULL;
 
 			args_write_line (f,
 			                 NMV_OVPN_TAG_SECRET,
 			                 nm_utils_str_utf8safe_unescape (value, &s_free),
-			                 _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION)));
+			                 nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION)));
 		}
 	}
 
@@ -1924,14 +1958,25 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_TUN_MTU, s_vpn, NM_OPENVPN_KEY_TUNNEL_MTU);
 
+	args_write_line_setting_value_int (f, NMV_OVPN_TAG_CONNECT_TIMEOUT, s_vpn, NM_OPENVPN_KEY_CONNECT_TIMEOUT);
+
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_FRAGMENT, s_vpn, NM_OPENVPN_KEY_FRAGMENT_SIZE);
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_FILE);
+	if (value)
+		args_write_line (f, NMV_OVPN_TAG_CRL_VERIFY, value);
+	else {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_DIR);
+		if (value)
+			args_write_line (f, NMV_OVPN_TAG_CRL_VERIFY, value, "dir");
+	}
+
 	{
 		gs_free char *device_free = NULL;
 		const char *device_type, *device;
 
-		device_type = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_DEV_TYPE));
-		device = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_DEV));
+		device_type = nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_DEV_TYPE));
+		device = nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_DEV));
 		device = nm_utils_str_utf8safe_unescape (device, &device_free);
 		args_write_line (f,
 		                 NMV_OVPN_TAG_DEV,
@@ -1956,8 +2001,8 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_PING_RESTART, s_vpn, NM_OPENVPN_KEY_PING_RESTART);
 
-	local_ip = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP));
-	remote_ip = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP));
+	local_ip = nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP));
+	remote_ip = nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP));
 	if (local_ip && remote_ip)
 		args_write_line (f, NMV_OVPN_TAG_IFCONFIG, local_ip, remote_ip);
 
@@ -1971,7 +2016,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		args_write_line_setting_value (f, NMV_OVPN_TAG_TLS_REMOTE, s_vpn, NM_OPENVPN_KEY_TLS_REMOTE);
 
 		x509_name =  nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_VERIFY_X509_NAME);
-		if (_arg_is_set (x509_name)) {
+		if (nmovpn_arg_is_set (x509_name)) {
 			const char *name;
 			gs_free char *type = NULL;
 
@@ -1986,26 +2031,33 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		}
 
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA);
-		if (_arg_is_set (key)) {
+		if (nmovpn_arg_is_set (key)) {
 			gs_free char *s_free = NULL;
 			args_write_line (f,
 			                 NMV_OVPN_TAG_TLS_AUTH,
 			                 nm_utils_str_utf8safe_unescape (key, &s_free),
-			                 _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR)));
+			                 nmovpn_arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR)));
 		}
 
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT);
-		if (_arg_is_set (key)) {
+		if (nmovpn_arg_is_set (key)) {
 			gs_free char *s_free = NULL;
 			args_write_line (f,
 			                 NMV_OVPN_TAG_TLS_CRYPT,
 			                 nm_utils_str_utf8safe_unescape (key, &s_free));
 		}
 
+		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_EXTRA_CERTS);
+		if (nmovpn_arg_is_set (key)) {
+			gs_free char *s_free = NULL;
+			args_write_line (f,
+			                 NMV_OVPN_TAG_EXTRA_CERTS,
+			                 nm_utils_str_utf8safe_unescape (key, &s_free));
+		}
 	}
 
 	proxy_type = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_TYPE);
-	if (_arg_is_set (proxy_type)) {
+	if (nmovpn_arg_is_set (proxy_type)) {
 		const char *proxy_server;
 		const char *proxy_port;
 		const char *proxy_retry;
